## Program to convert Access database table to MySQL create and INSERT statements
## that can be copied, pasted and run in MySQL server to create table.
## Greg Reid
## 27/07/2020

## File Preparation notes

## This program makes a few assumptions about the format of data in the csv file:
##    - a date is dd/mm/yyyy
##    - a time hh:mm:ss
##    - a boolean is TRUE or FALSE

## For the program to work the following preparation is recommended:
## 1. Open Access database and export table as Excel file.
## 2. Format any date cells to dd/mm/yyyy
## 3. Format time cells using 'custom' to hh:mm:ss
## 4. Save Excel file as csv file.

## Run the program and type in the name of your csv file (without the .csv).
## The program will create a text file with the CREATE and INSERT statements required to create and
## populate a MySQL database.

#Read in data from csv file into 2D list
def readfile(tableData,fileName):
    fileName = fileName + ".csv"
    with open(fileName) as tData:
        for each in tData.readlines():
            each = each[0:-1]
            temp = each.split(",")
            tableData.append(temp)
    return tableData


#Check for Boolean field
def checkForBoolean(tableData,element):
    #Search for column of 1s and 0s
    found10 = True
    for loop in range(1,len(tableData)-1):
         if tableData[loop][element] != "0" and tableData[loop][element] != "1":
            found10 = False
    #Search for column of TRUEs and FALSEs
    foundTF = True
    for loop in range(1,len(tableData)-1):
         if tableData[loop][element] != "TRUE" and tableData[loop][element] != "FALSE":
            foundTF = False

    if found10 or foundTF:
        return True
    else:
        return False


#Determine field types is date or time
def checkForDate(dataItem):
    if dataItem.count("/") == 2 and len(dataItem) == 10:
        return "date"
    else:
        return "other"


#Determine field types is date or time
def checkForTime(dataItem):
    if dataItem.count(":") == 2 and len(dataItem) == 8:
        return "time"
    else:
        return "other"


#Determine field types from first row of data
def checkForNumber(dataItem):
    count = 0
    for c in range(len(dataItem)):
        if (ord(dataItem[c:c+1]) >= 48 and ord(dataItem[c:c+1]) <= 57) or ord(dataItem[c:c+1]) == 46 or ord(dataItem[c:c+1]) == 45:
            count += 1

    if count == len(dataItem) and dataItem.count("-") <= 1 and dataItem.count(".") <= 1:
        #numbers found
        if dataItem.count("-") == 0 and dataItem.count(".") == 0:
            return "int"
        elif dataItem.count("-") == 1 and dataItem.index("-") == 0 and dataItem.count(".") == 0:
            return "int"
        elif dataItem.count(".") == 1 and dataItem.count("-") == 0:
            return "float(6,2)"
        elif dataItem.count(".") == 1 and dataItem.count("-") == 1 and dataItem.index("-") == 0:
            return "float(6,2)"


# Generate the create statement and write to new txt file (with same name as csv file).
def createTable(tableData,fileName,dataTypes,numFields):
    createText = "CREATE TABLE " + fileName + "(\n"

    #Create string containing SQL create table statement
    for loop in range(numFields):
        createText = createText + tableData[0][loop] + " " + dataTypes[loop] + " NOT NULL"
        if loop < numFields-1:
            createText = createText + ",\n"
        else:
            createText = createText + "\n"
    createText = createText + ");\n\n"

    newFile = fileName + ".txt"
    with open(newFile,'w') as createFile:
        createFile.write(createText)
        createFile.write("-- " + "The following may be added to the CREATE statement" + "\n")
        createFile.write("-- " + "Primary Key Syntax (added after field name and NOT NULL)" + "\n")
        createFile.write("-- " + "    PRIMARY KEY (fieldName)" + "\n")
        createFile.write("-- " + "         or for compound PK" + "\n")
        createFile.write("-- " + "    CONSTRAINT PK_compoundFieldName PRIMARY KEY (field1,field2)" + "\n")
        createFile.write("-- " + "Foreign Key Syntax (added after field list)" + "\n")
        createFile.write("-- " + "    FOREIGN KEY (field name) REFERENCES tableName(fieldName)" + "\n")
        createFile.write("-- " + "Text Field Max Size" + "\n")
        createFile.write("-- " + "    Edit (255) to required length" + "\n")
        createFile.write("-- " + "Validation of length of text (added after create statement as new statement" + "\n")
        createFile.write("-- " + "    CHECK (CHAR_LENGTH(fieldName) > value)" + "\n")
        createFile.write("-- " + "Range Check (added after field list)" + "\n")
        createFile.write("-- " + "    CHECK (fieldName >= value)" + "\n")
        createFile.write("-- " + "    CHECK (fieldName >= value AND fieldName <= value2)" + "\n")
        createFile.write("-- " + "Restricted choice" + "\n")
        createFile.write("-- " + "    CHECK(fieldName in ('value1','value2','value3')) " + "\n\n")
        createFile.write("-- " + "Note" + "\n")
        createFile.write("-- " + "At N5 Check constraints (that are ignored by a MySQL server) " + "\n")
        createFile.write("-- " + "are being used instead of triggers." + "\n\n")

# Use rows in csv to generate INSERT statements formatted for MySQL (date and bool formats changed)
def createInserts(tableData,fileName,dataTypes,numFields,numRecords):

    numFields = len(tableData[0])

    newFile = fileName + ".txt"
    with open(newFile,'a') as insertFile:
        for outer in range(1,numRecords):
            temp = "INSERT INTO " + fileName + " VALUES("
            for inner in range(numFields):
                if dataTypes[inner] == "varchar(255)" or dataTypes[inner] == "time":
                    temp = temp + '"' + tableData[outer][inner] + '"'
                elif dataTypes[inner] == "date":
                    dateparts = tableData[outer][inner].split("/")
                    temp = temp + '"' + dateparts[2] + "-" + dateparts[1] + "-" + dateparts[0] + '"'
                elif dataTypes[inner] == "bool":
                    if tableData[outer][inner] == "TRUE":
                        temp = temp + str(1)
                    else:
                        temp = temp + str(0)
                else:
                    temp = temp + tableData[outer][inner]
                if inner < numFields-1:
                    temp = temp + ","
            temp = temp + ");\n"
            insertFile.write(temp)



#MAIN PROGRAM

#Read file data
tableData = []
fileName = str(input("Please enter the name of the file"))
tableData = readfile(tableData,fileName)

numFields = len(tableData[0])
dataTypes = [""]*numFields
numRecords = len(tableData)

#Determine and store field types
for loop in range(numFields):
    booleanField = checkForBoolean(tableData,loop)
    if booleanField:
        dataTypes[loop] = "bool"
    else:
        dateFields = checkForDate(tableData[1][loop])
        if dateFields == "date":
            dataTypes[loop] = "date"
        else:
            timeFields = checkForTime(tableData[1][loop])
            if timeFields == "time":
                dataTypes[loop] = "time"
            else:
                numberField = checkForNumber(tableData[1][loop])
                if numberField == "int":
                    dataTypes[loop] = "int"
                elif numberField == "float(6,2)":
                    dataTypes[loop] = "float(6,2)"
                else:
                    dataTypes[loop] = "varchar(255)"

#Generate CREATE TABLE statement
createTable(tableData,fileName,dataTypes,numFields)

#Generate INSERT statements
createInserts(tableData,fileName,dataTypes,numFields,numRecords)

print("New File Created")